// ----------------------------------------------------------------------
// typemaps.i
// 
// This file defines all of the type-mappings for our language (Lua).  
// A typemap of 'SWIG_DEFAULT_TYPE' should be used to create default
// mappings.
// ----------------------------------------------------------------------

/**************************** FUNCTION INPUTS ****************************/

// Integers 
%typemap(in) int,
             short,
             long,
             unsigned int,
             unsigned short,
             unsigned long,
             signed char,
             unsigned char,
             float,
             double          
{
#ifdef LUA_MS
  $target = ($type) lua_getnumber (lua_state, $source);
#else
  $target = ($type) lua_getnumber ($source);
#endif
}

// Strings
%typemap(in) char *  
{
#ifdef LUA_MS
  $target = lua_getstring (lua_state, $source);
#else
  $target = lua_getstring ($source);
#endif
}

// void *
// User defined types and all other pointers

%typemap(in) void * SWIG_DEFAULT_TYPE,
             User * SWIG_DEFAULT_TYPE
{
#ifdef LUA_MS
  if (!lua_isnil (lua_state, $source))
    $target = lua_getuserdata (lua_state, $source);
  else $target = NULL;
#else
  if (!lua_isnil ($source)) $target = lua_getuserdata ($source);
  else $target = NULL;
#endif
}

/**************************** FUNCTION OUTPUTS ****************************/    

// Signed integers
%typemap(out) int         ,
              short       ,
              long        ,
              signed char ,
              unsigned       ,
              unsigned short ,
              unsigned long  ,
              unsigned char  ,
              double ,
              float  
{
#ifdef LUA_MS
  lua_pushnumber (lua_state, (double) $source);
#else
  lua_pushnumber ((double) $source);
#endif
}

// Strings
%typemap(out) char *
{
#ifdef LUA_MS
  lua_pushstring (lua_state, $source);
#else
  lua_pushstring ($source);
#endif
}

// Pointers
%typemap(out) void * SWIG_DEFAULT_TYPE, User * SWIG_DEFAULT_TYPE
{
#ifdef LUA_MS
  if ($source != NULL) lua_pushuserdata (lua_state, $source);
  else lua_pushnil (lua_state);
#else
  if ($source != NULL) lua_pushuserdata ($source);
  else lua_pushnil ();
#endif
}

/**************************** VARIABLE CREATION ****************************/    

// Integers
// Doubles

%typemap(varinit) int          ,
                  unsigned int ,
                  double 
{
#ifdef LUA_MS
  lua_pushnumber (lua_state, $source);
  lua_setglobal (lua_state, "$target");
#else
  lua_pushnumber ($source);
  lua_setglobal ("$target");
#endif
}

// Strings

%typemap(varinit) char *  {
#ifdef LUA_MS
  lua_pushstring (lua_state, $source);
  lua_setglobal (lua_state, "$target");
#else
  lua_pushstring ($source);
  lua_setglobal ("$target");
#endif
}

/****************************** CONSTANTS **********************************/    

// Signed Integers
%typemap(const) int           ,
                short         ,
                long          ,
                signed char   ,
                unsigned        ,
                unsigned short  ,
                unsigned long   ,
                unsigned char   ,
                double ,
                float  
{
#ifdef LUA_MS
  lua_pushnumber (lua_state, $source);
  lua_setglobal (lua_state, "$target");
#else
  lua_pushnumber ($source);
  lua_setglobal ("$target");
#endif
}

// Strings
%typemap(const) char *
{
#ifdef LUA_MS
  lua_pushstring (lua_state, $source);
  lua_setglobal (lua_state, "$target");
#else
  lua_pushstring ($source);
  lua_setglobal ("$target");
#endif
}
  
// Pointers
%typemap(const) void * SWIG_DEFAULT_TYPE,
                User * SWIG_DEFAULT_TYPE
{
#ifdef LUA_MS
  lua_pushuserdata (lua_state, $source);
  lua_setglobal (lua_state, "$target");
#else
  lua_pushuserdata ($source);
  lua_setglobal ("$target");
#endif
}

